#include "CifFile.h"
#include "CifParserBase.h"
#include "CifFileReadDef.h"
#include <iostream>

void update_attribute_value(CifFile *fobj, const string& blockName, const string& category, const string& attribute, const string& value);
void  update_entry_ids(CifFile *fobj, const string& blockName, const string& idName);
string get_attribute_value(CifFile *fobj, const string& blockName, const string& category, const string& attribute);
void FillTestTable(ISTable *s);

#define FALSE 0
#define defdb "DEPOSIT-0"

int main() {

  string diags;
  CifFile * fobjR = NULL;
  string blockName;
  string dbname;

  CifParser* cifParserR = NULL;


  fobjR = new CifFile(CREATE_MODE, "./test2.sdb", true);

  cifParserR = new CifParser(fobjR, fobjR->GetVerbose());

  cifParserR->Parse("./Test2.cif", diags);

  delete(cifParserR);

  fobjR->Close();
  if (fobjR) delete fobjR;
  fobjR = new CifFile(UPDATE_MODE, "./test2.sdb");

  blockName = fobjR->GetFirstBlockName();
  // VLAD: WHY DO WE USE _database.ndb_code_NDB. Shouldn't we be using
  // ndb_code_NDB?
  dbname=get_attribute_value(fobjR,blockName,"database","_database.ndb_code_NDB");
  if (dbname.empty()) dbname=get_attribute_value(fobjR,blockName,"database","_database.ndb_code_PDB");
  if (dbname.empty()) {
    dbname = defdb;
  }

  fobjR->Write("./YYtest.ocif");  
 
  update_entry_ids(fobjR,blockName, dbname);
#ifdef VLAD_DEBUG
  // Renaming to itself. Not needed and creates some problems.
#endif
  fobjR->RenameFirstBlock(dbname); 
  fobjR->Close();
  if (fobjR) delete fobjR;

  fobjR = new CifFile(UPDATE_MODE, "./test2.sdb");
  fobjR->Write("./ZZtest.ocif"); 
  fobjR->Close();
  if (fobjR) delete fobjR;

  exit(0);
}

string get_attribute_value(CifFile *fobj, const string& blockName, const string& category, const string& attribute)
{
  ISTable *t=NULL;

  vector<string> col;
  int nRow;
  string ret;


  Block& block = fobj->GetBlock(blockName);

  if (block.IsTablePresent(category)) {
    t=block.GetTablePtr(category);
    nRow = t->GetNumRows();
    if (nRow > 0) {
      if (t->IsColumnPresent(attribute)) {
  t->GetColumn(col, attribute);
  if (!col.empty() && (col[0].size() > 0) ) {
 ret = col[0];
  }
      }
    }
  }

  return(ret);

}


void update_attribute_value(CifFile *fobj, const string& blockName, const string& category, 
    const string& attribute, const string& value)
{
  ISTable *t=NULL;

  string cs;
  vector<string> uv;
  int nRow;
  int i;

  if ((blockName.empty()) || (category.empty()) || (attribute.empty()) || (value.empty()) ) return;

  cs.clear(); cs = value;

  Block& block = fobj->GetBlock(blockName);

  if ((t=block.GetTablePtr(category)) != NULL)
  {
    nRow = t->GetNumRows();
    if (nRow > 0)
    {
      if (t->IsColumnPresent(attribute))
      {
        uv.clear();
        for (i=0; i< nRow; i++)
        {
            uv.push_back(cs);
        }
        t->ClearColumn(attribute);
        t->FillColumn(attribute, uv);
        block.WriteTable(t);
      }
    }
  }
}

void update_attribute_value_bug(CifFile *fobj, const string& blockName, const string& category, 
 const string& attribute, const string& value)
{
  ISTable *t=NULL;
  string cs;
  string us;
  int nRow;
  int i;

  if ((blockName.empty()) || (category.empty()) || (attribute.empty()) || (value.empty()) ) return;
  
  Block& block = fobj->GetBlock(blockName);

  if ((t=block.GetTablePtr(category)) != NULL) {
    nRow = t->GetNumRows();
    if (nRow > 0) {
      if (t->IsColumnPresent(attribute)) {

         for (i=0; i< nRow; i++) {
                         cs=value;
                     t->UpdateCell(i, attribute, cs);
                   }
                   cout << (*t);
      }
    }
  }
}

void  update_entry_ids(CifFile *fobj, const string& blockName, const string& idName)
{
  update_attribute_value(fobj,blockName,"entry","id",idName);    
  update_attribute_value(fobj,blockName,"struct_keywords","entry_id",idName);    
  update_attribute_value(fobj,blockName,"database","entry_id",idName);    
  update_attribute_value(fobj,blockName,"database","ndb_code_NDB",idName);    
  update_attribute_value(fobj,blockName,"exptl","entry_id",idName);    
  
  update_attribute_value(fobj,blockName,"struct","entry_id",idName);    
  update_attribute_value(fobj,blockName,"ndb_na_struct_keywds","entry_id",idName);    
  update_attribute_value(fobj,blockName,"ndb_database_status","entry_id",idName);    
  update_attribute_value(fobj,blockName,"ndb_database_proc","entry_id",idName);    
  update_attribute_value(fobj,blockName,"ndb_coord","entry_id",idName);    
  update_attribute_value(fobj,blockName,"reflns","entry_id",idName);    
  update_attribute_value(fobj,blockName,"computing","entry_id",idName);    
  update_attribute_value(fobj,blockName,"symmetry","entry_id",idName);    
  update_attribute_value(fobj,blockName,"cell","entry_id",idName);    
  update_attribute_value(fobj,blockName,"refine","entry_id",idName);    
  update_attribute_value(fobj,blockName,"refine_analyze","entry_id",idName);    
  update_attribute_value(fobj,blockName,"ndb_refine","entry_id",idName); 
  update_attribute_value(fobj,blockName,"atom_sites","entry_id",idName);    
}


